/*
 * E.S.O. - VLT project 
 *
 * "@(#) $Id: BiasData.C,v 1.1.1.1 2002/04/04 20:11:52 brighton Exp $" 
 *
 * BiasData.C - member functions for class BiasData
 *
 * BiasData is a class for mananging the bias data which can be
 * subtracted from the image data
 * 
 * who             when      what
 * --------------  --------  ----------------------------------------
 * P. Biereichel   22/03/99  Created
 */

#include "BiasData.h"
#include "error.h"
 
BiasData::BiasData()
    :   biasImage_(NULL),
	idxBias_(0)
{
    for (int i=0; i< MAXBIAS; i++) {
	biasImages_[i] = NULL;
	files_[i][0] = '\0';
    }
    clear(0);
}

/*
 * return status: selected bias image !loaded, on or off
 */
int BiasData::status() 
{
    if (! biasImage_)
	return -1; 
    if (bias.on)
	return 1;
    return 0;
}

/*
 * switch bias subtraction on
 */
int BiasData::on()
{
    if (! biasImage_) {
	error("selected bias image not loaded");
	return 1;
    }
    bias.on = 1;
    return 0;
}

/*
 * switch bias subtraction off
 */
int BiasData::off()
{
    if (! bias.on)
	return 1;
    bias.on = 0;
    return 0;
}

/*
 * return filename or objectname of a bias image
 */
char* BiasData::file(int nr)
{
    if (nr < 0 || nr >=MAXBIAS)
	return '\0';
    return &files_[nr][0];
}

/*
 * load bias frame from file
 */
int BiasData::file(char *file, int nr)
{
    // if not '-' (stdin) check that it is a file
    if (strcmp(file, "-") != 0) { 
	struct stat buf;
	if (stat(file, &buf) != 0 || S_ISREG(buf.st_mode) == 0) {
	    error("expected a file, but got: ", file);
	    return 1;
	}
    }
    int on = bias.on;
    clear(nr);
    // read the FITS image 
    biasImages_[nr]  = ImageData::makeImage(BIASNAME, FitsIO::read(file), &bias, 0);
    if (! biasImages_[nr])
	return 1;
    strcpy(&files_[nr][0], file);
    if (nr == idxBias_) {
	bias.on = on;
	select(nr);
    }
    return 0;
}

/*
 * copy image to a bias frame
 */
int BiasData::copy(ImageData* image, char* filename, int nr)
{
    if (! image || nr < 0 || nr >=MAXBIAS)
	return 1;
    int on = bias.on;
    clear(nr);

    int length = image->data().length();
    Mem data(length, 0), header;
    if (data.status() != 0)
	return 1;
    FitsIO* fits = new FitsIO(image->width(), image->height(), image->dataType(), 
			      0.0, 1.0, header, data);
    if (!fits || fits->status() != 0)
	return 1;

    biasImages_[nr] = ImageData::makeImage(BIASNAME, fits, &bias, 0);
    if (! biasImages_[nr])
	return 1;

    memcpy((char *)data.ptr(), (char *)image->data().ptr(), length);

    biasImages_[nr]->object(image->object());
    strcpy(&files_[nr][0], filename);
    if (nr == idxBias_) {
	bias.on = on;
	select(nr);
    }
    return 0;
}

/*
 * select a bias frame
 */
int BiasData::select(int nr)
{
    if (nr < 0 || nr >= MAXBIAS)
	return 1;
    idxBias_ = nr;
    if (biasImages_[nr] != NULL) {
	biasImage_  = biasImages_[nr];
	bias.ptr    = (char *)biasImage_->image().dataPtr();
	bias.width  = biasImage_->image().width();
	bias.height = biasImage_->image().height();
	bias.type   = biasImage_->dataType();
    }
    else
	clear(nr);

    return 0;
}

/*
 * return number for selected bias frame
 */
int BiasData::select()
{
    if (! biasImages_[idxBias_])
	return -1;
    return idxBias_;
}

/*
 * clear bias frame
 */
void BiasData::clear(int nr)
{
    if (nr < 0 || nr >=MAXBIAS)
	return;
    if (nr == idxBias_) {
	biasImage_   = NULL;
	bias.on      = 0;
	bias.ptr     = NULL;
	bias.width   = 0;
	bias.height  = 0;
	bias.type    = -1;
    }
    files_[nr][0] = '\0';
    if (biasImages_[nr]) {
	delete biasImages_[nr];
	biasImages_[nr] = NULL;
    }
}
